using System;
using System.IO;
using DHI.PFS;
using DHI.Generic.MikeZero.DFS;
using DHI.Generic.MikeZero.DFS.dfs123;

namespace DHI.Generic.MikeZero.DFSTest
{
  /// <summary>
  /// Creates a dfs2 file with items based on information from a dfs1 file (e.g. to emulate a simple 2D flather condition file)
  /// The dfs1 file is assumed to have an equidistant axis
  /// The extension of dfs1 file defines the x-axis, the y axis is described by number of layers and spacing
  /// Values are read from dfs1 file and constant over depth
  /// </summary>
  /// <param name="pfsfilePath">File and path to the pfsExample.pfs file</param>
  
  class CreateDfs2FromDfs1
  {
	       /// <summary>
      /// Static constructor, setting up search paths for MIKE assemblies
      /// </summary>
      static CreateDfs2FromDfs1()
      {
        // The setup method will make your application find the MIKE assemblies at runtime.
        if (!DHI.Mike.Install.MikeImport.Setup(23, DHI.Mike.Install.MikeProducts.Mike1D))
          throw new Exception("Could not find a MIKE installation");
      }

    private string _inputfilename;
    private string _outputfilename;
    private int _noYlevels;
    private double _dYspacing;
    
	
    /// <summary>
    /// Main method, main entry to program
    /// </summary>
    public static void Main(string[] args)
    {
         if (args.Length > 1)
         {
            Console.Out.WriteLine("Create dfs2 file from dfs1 file, constant values along y-axis");
            Console.Out.WriteLine("Usage:");
            Console.Out.WriteLine("   CreateDfs2FromDfs1.exe inputfile.pfs");
            return;
         }

         string filename = args[0];
         
         // Read information from pfs-file
         CreateDfs2FromDfs1 converter = new CreateDfs2FromDfs1();
         converter.ReadPFSfile(filename);

         // check input and extract main parameters
         int ierr = 0;
         converter.WritePFSextract(ierr);
         if (ierr == 0)
         { 
            // process data
            converter.Process();
         }
   
    }

      
    /// <summary>  ReadPFSfile
    /// Reads pfs-input file and extracts info 
    /// Loading/Reading the file
    /// <param name="filename">input pfs file name</param>
    /// </summary>
    public void ReadPFSfile (string filename)
    {
      PFSFile pfsFile = new PFSFile(filename, false);

      // Outmost section
      PFSSection target = pfsFile.GetTarget("CREATE_DFS2_FROM_DFS1", 1);
    
      // Initialize
      string inputfilename = "input.dfs1";
      string outputfilename = "output.dfs2";
      int noLevels = 2;
      double spacing = 1.0;
      
      // Get input parameters
      inputfilename = target.GetKeyword("Input_File_Name", 1).GetParameter(1).ToString();
      outputfilename = target.GetKeyword("Output_File_Name", 1).GetParameter(1).ToString();
      noLevels = target.GetKeyword("Number_of_Layers", 1).GetParameter(1).ToInt();
      spacing = target.GetKeyword("Spacing", 1).GetParameter(1).ToDouble();
      
      // save input parameters 
      _inputfilename = inputfilename;
      _outputfilename = outputfilename;
      _noYlevels  = noLevels;
      _dYspacing = spacing;
   }
   
    /// <summary> WritePFSextract
    /// Writes extracted data from pfs-input to check
    /// <param name="ierr">stop in case of errors</param>
    /// </summary>
    public void WritePFSextract (int ierr)
    {
        Console.WriteLine("Input file = {0}", _inputfilename);
        Console.WriteLine("Output file = {0}", _outputfilename);
        Console.WriteLine("Number of layers = {0}", _noYlevels);
        Console.WriteLine("Spacing of layers = {0}", _dYspacing);

        // Check output
        {
            string fileext = Path.GetExtension(_inputfilename);
            if (fileext != ".dfs1")
            {
                Console.WriteLine("ERR: Input file name = {0} does not have the extension .dfs1", _inputfilename);
                Console.WriteLine("Press any key to exit");
                Console.ReadKey(); 
                ierr = 1;
            }

            fileext = Path.GetExtension(_outputfilename);
            if (fileext != ".dfs2")
            {
                Console.WriteLine("ERR: Output file name = {0} does not have the extension .dfs2", _outputfilename);
                Console.WriteLine("Press any key to exit");
                Console.ReadKey(); 
                ierr = 1;
            }
        }
    }


    /// <summary> Process
    /// Main routines to process information
    /// </summary>
    public void Process()
    {
        // Open the input file as a dfs1 file 
        Dfs1File dfs1File = DfsFileFactory.Dfs1FileOpen(_inputfilename);

        if (dfs1File.SpatialAxis.AxisType == SpaceAxisType.EqD1)
        {
            // Spatial axis for this file is a 1D equidistant axis 
            IDfsAxisEqD1 axisEqD1 = ((IDfsAxisEqD1)dfs1File.SpatialAxis);
        }
     
        if (dfs1File.SpatialAxis.AxisType == SpaceAxisType.NeqD1)
        {
            // Spatial axis is a non-equidistant axis - not valid!
            Console.WriteLine("ERR: Input file = {0} are defined by non-eq spatial axis", _inputfilename);
        }
     
        int numItems = dfs1File.ItemInfo.Count;

        // Create a new dfs2 file 
        DfsFactory factory = new DfsFactory();
        Dfs2Builder builder = Dfs2Builder.Create("", "MIKE SDK", 0);

        // Header information is contained in the IDfsFileInfo
        IDfsFileInfo fileInfo = dfs1File.FileInfo;
        builder.SetDataType(0);

        // Get and set the projection 
        IDfsProjection myproj = fileInfo.Projection;
        string projectionString = myproj.WKTString;   
        double xorigo = myproj.Longitude;
        double yorigo = myproj.Latitude;
        double orientation = myproj.Orientation;
        builder.SetGeographicalProjection(factory.CreateProjectionGeoOrigin(projectionString, xorigo, yorigo, orientation));

        // Get and set the time step
        builder.SetTemporalAxis(fileInfo.TimeAxis); 
        int noTSsteps = fileInfo.TimeAxis.NumberOfTimeSteps;

        // Set delete values
        builder.DeleteValueByte = fileInfo.DeleteValueByte;
        builder.DeleteValueDouble = fileInfo.DeleteValueDouble;
        builder.DeleteValueFloat = fileInfo.DeleteValueFloat;
        builder.DeleteValueInt = fileInfo.DeleteValueInt;
        builder.DeleteValueUnsignedInt = fileInfo.DeleteValueUnsignedInt;

        // Get/Set spatial axis 
        IDfsAxisEqD1 myaxisEqD1 = ((IDfsAxisEqD1)dfs1File.SpatialAxis);
        int NoX = myaxisEqD1.XCount;
        double dx = myaxisEqD1.Dx;
        builder.SetSpatialAxis(factory.CreateAxisEqD2(eumUnit.eumUmeter, NoX, 0, dx, _noYlevels, 0, _dYspacing));

        // Set up dynamic items
        IDfsSimpleDynamicItemInfo dynamicItemInfo;
        for (int j = 0; j < numItems; j++)
        {
            dynamicItemInfo = dfs1File.ItemInfo[j];
            builder.AddDynamicItem(dynamicItemInfo.Name, dynamicItemInfo.Quantity, DfsSimpleType.Float, DataValueType.Instantaneous);
        }
            
        // Create file
        builder.CreateFile(_outputfilename);

        // Get the file
        Dfs2File target = builder.GetFile();

        // Loop over all time steps
        for (int i = 0; i < noTSsteps; i++)
        {
            {
               // Loop over all items, read time step and extract value
               for (int j = 0; j < numItems; j++)
               {
                   IDfsItemData<float> sourceItemData = dfs1File.ReadItemTimeStepNext() as IDfsItemData<float>;
                   double itemtime = sourceItemData.Time;

                   // Create empty item data, and copy over data from source
                   IDfsItemData2D<float> itemData2D = (IDfsItemData2D<float>)target.CreateEmptyItemData(j+1);

                   for (int k = 0; k < sourceItemData.Data.Length; k++)
                   { 
                       for (int l = 0; l < _noYlevels; l++)
                       {
                           itemData2D[k, l] = sourceItemData.Data[k];
                       }
                   }

                   // write values to file
                   target.WriteItemTimeStep(j+1, i, itemtime, itemData2D.Data);
               }
           }
        }
        dfs1File.Close();
        target.Close();  
    }
  }  
}
